/**

	Memory Controller Test
	
	File			:	Controller Definition

	Created by		:	Reznichenko Anton aka AlterVision
						mailto:me@av13.ru
						http://www.av13.ru

	Project started :	30 november 2008 - 17:00
	Last edited		:	30 november 2008 - 17:25

	Uses			:	<windows.h> kernel32.dll

	Describtion		:
		
		Memory controller is used to allocate memory for the application
		and replace default malloc and free functions

		It uses bitmap storage algorithm to keep informaion about memory,
		stored in 8-byte blocks.
		
*/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <malloc.h>
#include "mc.h"

#define TEST_PROCEDURES 10000

typedef struct _operation {
	int		kill;
	int		killed;
	size_t	size;
} operation_t;

operation_t		operations [TEST_PROCEDURES];
void *			test [TEST_PROCEDURES];

int main ( void ) {

	int				i, r, k, tp;
	LARGE_INTEGER	st, en;
	size_t			variable_size, tot;
	size_t			mins, maxs;

	printf("Memory Controller Functions - Test Field\n");
	
	/* ********************************************************************** */
	/* ********************************************************************** */
	/* ********************************************************************** */

	// Ultra Random Algorithm
	for (tp = 1; tp <= 2; tp++) {

		printf("Ultra-Random Test - generating ... ");

		tot = maxs = 0;
		mins = 100;

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			// Generate Random Size
			r = rand() % 30;
			switch (r) {
				case 7:
					variable_size = ((rand()%120)+9) * 8192;				
					break;

				case 13:
					variable_size = ((rand()%120)+9) * 4096;
					break;

				case 10:
				case 11:
					variable_size = ((rand()%120)+9) * 512;
					break;

				case 20:
				case 21:
				case 22:
					variable_size = ((rand()%60)+5) * 64;
					break;

				case 19:
				case 18:
				case 17:
					variable_size = ((rand()%30)+3) * 4;
					break;
				
				default:
					variable_size = (rand() % 7) + 1;

			}

			operations[i].size = variable_size;
			if (variable_size < mins) mins = variable_size;
			if (variable_size > maxs) maxs = variable_size;
			operations[i].killed = 0;
			operations[i].kill = 0;
			if ((rand () % 3) == 0 && i) {
				k = rand () % i;
				if (operations[k].killed == 0) {
					operations[i].kill = k;
					operations[k].killed = 1;
				}
			}
			tot += variable_size;
		}

		printf(" %dmb from %db to %dkb ... Ok!\n", tot / 1048576, mins, maxs / 1024);

		/* ************************************************************* */
		/* Malloc Block Start */

		printf("Function: windows malloc - in progress ...\n");

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			test[i] = malloc(operations[i].size);
			if (operations[i].kill) {
				free (test[operations[i].kill]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Mixed: %16d\n", (en.QuadPart - st.QuadPart));

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			if (operations[i].killed == 0) {
				free (test[i]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Free:  %16d\n", (en.QuadPart - st.QuadPart));

		/* ************************************************************** */


		/* ************************************************************* */
		/* AlterVision [mc] Block Start */

		printf("Function: mc_alloc - in progress ...\n");

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			test[i] = mc_alloc(operations[i].size);
			if (operations[i].kill) {
				mc_delete (test[operations[i].kill]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Mixed: %16d\n", (en.QuadPart - st.QuadPart));

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			if (operations[i].killed == 0) {
				mc_delete (test[i]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Free:  %16d\n", (en.QuadPart - st.QuadPart));

		/* ************************************************************** */

	}

	/* ********************************************************************** */
	/* ********************************************************************** */
	/* ********************************************************************** */

	// Sequential Random Algorithm
	for (tp = 1; tp <= 2; tp++) {

		printf("Sequential Test - generating ... ");

		tot = maxs = 0;
		mins = 100;

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			// Generate Random Size
			r = rand() % 30;
			switch (r) {
				case 7:
					variable_size = ((rand()%120)+9) * 8192;				
					break;

				case 13:
					variable_size = ((rand()%120)+9) * 4096;
					break;

				case 10:
				case 11:
					variable_size = ((rand()%120)+9) * 512;
					break;

				case 20:
				case 21:
				case 22:
					variable_size = ((rand()%60)+5) * 64;
					break;

				case 19:
				case 18:
				case 17:
					variable_size = ((rand()%30)+3) * 4;
					break;
				
				default:
					variable_size = (rand() % 7) + 1;

			}

			operations[i].size = variable_size;
			if (variable_size < mins) mins = variable_size;
			if (variable_size > maxs) maxs = variable_size;
			tot += variable_size;
		}

		printf(" %dmb from %db to %dkb ... Ok!\n", tot / 1048576, mins, maxs / 1024);

		/* ************************************************************* */
		/* Malloc Block Start */

		printf("Function: windows malloc - in progress ...\n");

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			test[i] = malloc(operations[i].size);
		}

		QueryPerformanceCounter (&en);
		printf(" Alloc: %16d\n", (en.QuadPart - st.QuadPart));

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			free (test[i]);
		}

		QueryPerformanceCounter (&en);
		printf(" Free:  %16d\n", (en.QuadPart - st.QuadPart));

		/* ************************************************************** */


		/* ************************************************************* */
		/* AlterVision [mc] Block Start */

		printf("Function: mc_alloc - in progress ...\n");

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			test[i] = mc_alloc(operations[i].size);
			if (operations[i].kill) {
				mc_delete (test[operations[i].kill]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Mixed: %16d\n", (en.QuadPart - st.QuadPart));

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			if (operations[i].killed == 0) {
				mc_delete (test[i]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Free:  %16d\n", (en.QuadPart - st.QuadPart));

		/* ************************************************************** */

	}
	/* ********************************************************************** */
	/* ********************************************************************** */
	/* ********************************************************************** */

	// Small Blocks Algorithm
	for (tp = 1; tp <= 2; tp++) {

		printf("Small Blocks Test - generating ... ");

		tot = maxs = 0;
		mins = 100;

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			// Generate Random Size
			variable_size = (rand() % 16) + 1;
			operations[i].size = variable_size;
			if (variable_size < mins) mins = variable_size;
			if (variable_size > maxs) maxs = variable_size;
			tot += variable_size;
		}

		printf(" %dkb from %db to %db ... Ok!\n", tot / 1024, mins, maxs );

		/* ************************************************************* */
		/* Malloc Block Start */

		printf("Function: windows malloc - in progress ...\n");

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			test[i] = malloc(operations[i].size);
		}

		QueryPerformanceCounter (&en);
		printf(" Alloc: %16d\n", (en.QuadPart - st.QuadPart));

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			free (test[i]);
		}

		QueryPerformanceCounter (&en);
		printf(" Free:  %16d\n", (en.QuadPart - st.QuadPart));

		/* ************************************************************** */


		/* ************************************************************* */
		/* AlterVision [mc] Block Start */

		printf("Function: mc_alloc - in progress ...\n");

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			test[i] = mc_alloc(operations[i].size);
			if (operations[i].kill) {
				mc_delete (test[operations[i].kill]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Mixed: %16d\n", (en.QuadPart - st.QuadPart));

		QueryPerformanceCounter (&st);

		for (i = 0; i < TEST_PROCEDURES; ++i) {
			if (operations[i].killed == 0) {
				mc_delete (test[i]);
			}
		}

		QueryPerformanceCounter (&en);
		printf(" Free:  %16d\n", (en.QuadPart - st.QuadPart));

		/* ************************************************************** */

	}

	printf("Finished ... Press any key ...");
	_getch();

	return 0;

}